@@ -1,8 +1,7 @@
-/*  $Id: BatrachiansEngine.cpp,v 1.45 2017/12/09 18:26:16 sarrazip Exp $
-    BatrachiansEngine.cpp - Main engine
+/*  BatrachiansEngine.cpp - Main engine
 
     batrachians - A fly-eating frog game.
-    Copyright (C) 2004-2011 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2004-2024 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -29,54 +28,6 @@
 #include <sstream>
 #include <algorithm>
 
-#include <time.h>
-
-#include "images/sky.xpm"
-#include "images/water.xpm"
-#include "images/lilypads0.xpm"
-#include "images/cloud0_0.xpm"
-#include "images/cloud1_0.xpm"
-#include "images/frog0_r_stand.xpm"
-#include "images/frog0_r_swim.xpm"
-#include "images/frog0_l_stand.xpm"
-#include "images/frog0_l_swim.xpm"
-#include "images/frog1_r_stand.xpm"
-#include "images/frog1_r_swim.xpm"
-#include "images/frog1_l_stand.xpm"
-#include "images/frog1_l_swim.xpm"
-#include "images/crosshairs.xpm"
-#include "images/fly0_r0.xpm"
-#include "images/fly1_r0.xpm"
-#include "images/fly2_r0.xpm"
-#include "images/fly3_r0.xpm"
-#include "images/tongue0_r2.xpm"
-#include "images/splash0.xpm"
-#include "images/splash1.xpm"
-#include "images/star0.xpm"
-#include "images/star1.xpm"
-
-#include "images/digit_user_0.xpm"
-#include "images/digit_user_1.xpm"
-#include "images/digit_user_2.xpm"
-#include "images/digit_user_3.xpm"
-#include "images/digit_user_4.xpm"
-#include "images/digit_user_5.xpm"
-#include "images/digit_user_6.xpm"
-#include "images/digit_user_7.xpm"
-#include "images/digit_user_8.xpm"
-#include "images/digit_user_9.xpm"
-
-#include "images/digit_computer_0.xpm"
-#include "images/digit_computer_1.xpm"
-#include "images/digit_computer_2.xpm"
-#include "images/digit_computer_3.xpm"
-#include "images/digit_computer_4.xpm"
-#include "images/digit_computer_5.xpm"
-#include "images/digit_computer_6.xpm"
-#include "images/digit_computer_7.xpm"
-#include "images/digit_computer_8.xpm"
-#include "images/digit_computer_9.xpm"
-
 using namespace std;
 using namespace flatzebra;
 
@@ -87,6 +38,9 @@ using namespace flatzebra;
 const int PAUL = 12;
 
 
+static const string pkgPixmapDir = GameEngine::getDirPathFromEnv(PKGPIXMAPDIR, "PKGPIXMAPDIR");
+
+
 ostream &
 operator << (ostream &out, const RCouple &c)
 {
@@ -165,26 +119,6 @@ deleteSprites(Container &c)
     c.clear();
 }
 
-static
-string
-getDir(const char *defaultValue, const char *envVarName)
-/*
-    Makes sure that the returned directory name ends with a slash.
-*/
-{
-    string dir;
-    const char *s = getenv(envVarName);
-    if (s != NULL)
-        dir = s;
-    else
-        dir = defaultValue;
-
-    if (!dir.empty() && dir[dir.length() - 1] != '/')
-        dir += '/';
-
-    return dir;
-}
-
 
 enum { SCRNWID = 800, SCRNHT = 540 };
 
@@ -201,11 +135,13 @@ BatrachiansEngine::BatrachiansEngine(const string &windowManagerCaption,
                                         size_t _maxNumFlies,
                                         bool _useSound,
                                         bool fullScreen,
-                                        bool processActiveEvent)
+                                        bool processActiveEvent,
+                                        bool useAcceleratedRendering)
   : GameEngine(Couple(SCRNWID, SCRNHT),
                 windowManagerCaption,
                 fullScreen,
-                processActiveEvent),  // may throw string exception
+                processActiveEvent,
+                useAcceleratedRendering),  // may throw string exception
     firstTime(true),
     tickCount(0),
     gameOver(true),
@@ -213,16 +149,16 @@ BatrachiansEngine::BatrachiansEngine(const string &windowManagerCaption,
     totalPauseTime(0),
     pauseStartTime(0),
 
-    blackColor(       SDL_MapRGB(theSDLScreen->format, 0x00, 0x00, 0x00)),
+    blackColor(mapRGBA(0x00, 0x00, 0x00)),
 
-    lilyPadSurface(NULL),
+    lilyPadTexture(NULL),
     lilyPadSurfacePos(),
     leftLilyPad( SCRNWID * 14 / 100, SCRNWID * 32 / 100),
     rightLilyPad(SCRNWID * 59 / 100, SCRNWID * 27 / 100),
 
-    skySurface(NULL),
-    waterSurface(NULL),
-    cloudSurfaces(),
+    skyTexture(NULL),
+    waterTexture(NULL),
+    cloudTextures(),
 
     starPA(2),
     stars(),
@@ -259,35 +195,35 @@ BatrachiansEngine::BatrachiansEngine(const string &windowManagerCaption,
     sounds(),
     fontDim(getFontDimensions())
 {
-    skySurface = IMG_ReadXPMFromArray(const_cast<char **>(sky_xpm));
-    if (skySurface == NULL)
+    skyTexture = createTextureFromFile(pkgPixmapDir + "sky.xpm");
+    if (skyTexture == NULL)
         throw PixmapLoadError(PixmapLoadError::UNKNOWN, NULL);
-    waterSurface = IMG_ReadXPMFromArray(const_cast<char **>(water_xpm));
-    if (waterSurface == NULL)
+    waterTexture = createTextureFromFile(pkgPixmapDir + "water.xpm");
+    if (waterTexture == NULL)
         throw PixmapLoadError(PixmapLoadError::UNKNOWN, NULL);
 
 
     /*  Lily pads:
     */
-    lilyPadSurface = IMG_ReadXPMFromArray(const_cast<char **>(lilypads0_xpm));
-    if (lilyPadSurface == NULL)
+    lilyPadTexture = createTextureFromFile(pkgPixmapDir + "lilypads0.xpm");
+    if (lilyPadTexture == NULL)
         throw PixmapLoadError(PixmapLoadError::UNKNOWN, NULL);
 
-    Couple size(lilyPadSurface->w, lilyPadSurface->h);
+    Couple size = getTextureSize(lilyPadTexture);
     lilyPadSurfacePos = Couple((SCRNWID - size.x) / 2, SCRNHT - size.y);
     int yWater = getYWater();
 
 
     /* Clouds:
     */
-    cloudSurfaces[0] = IMG_ReadXPMFromArray(const_cast<char **>(cloud0_0_xpm));
-    cloudSurfaces[1] = IMG_ReadXPMFromArray(const_cast<char **>(cloud1_0_xpm));
+    cloudTextures[0] = createTextureFromFile(pkgPixmapDir + "cloud0_0.xpm");
+    cloudTextures[1] = createTextureFromFile(pkgPixmapDir + "cloud1_0.xpm");
 
 
     /*  Stars:
     */
-    loadPixmap(star0_xpm, starPA, 0);
-    loadPixmap(star1_xpm, starPA, 1);
+    loadPixmap("star0.xpm", starPA, 0);
+    loadPixmap("star1.xpm", starPA, 1);
     for (int i = 20; i > 0; i--)
         stars.push_back(new RSprite(starPA,
                         RCouple(rand() % (SCRNWID - 100) + 50,
@@ -298,14 +234,14 @@ BatrachiansEngine::BatrachiansEngine(const string &windowManagerCaption,
 
     /*  User and computer frogs:
     */
-    loadPixmap(frog0_l_stand_xpm, frogPA, FROG_L_STAND);
-    loadPixmap(frog0_l_swim_xpm,  frogPA, FROG_L_SWIM);
-    loadPixmap(frog0_r_stand_xpm, frogPA, FROG_R_STAND);
-    loadPixmap(frog0_r_swim_xpm,  frogPA, FROG_R_SWIM);
-    loadPixmap(frog1_l_stand_xpm, frogPA, 4 + FROG_L_STAND);
-    loadPixmap(frog1_l_swim_xpm,  frogPA, 4 + FROG_L_SWIM);
-    loadPixmap(frog1_r_stand_xpm, frogPA, 4 + FROG_R_STAND);
-    loadPixmap(frog1_r_swim_xpm,  frogPA, 4 + FROG_R_SWIM);
+    loadPixmap("frog0_l_stand.xpm", frogPA, FROG_L_STAND);
+    loadPixmap("frog0_l_swim.xpm",  frogPA, FROG_L_SWIM);
+    loadPixmap("frog0_r_stand.xpm", frogPA, FROG_R_STAND);
+    loadPixmap("frog0_r_swim.xpm",  frogPA, FROG_R_SWIM);
+    loadPixmap("frog1_l_stand.xpm", frogPA, 4 + FROG_L_STAND);
+    loadPixmap("frog1_l_swim.xpm",  frogPA, 4 + FROG_L_SWIM);
+    loadPixmap("frog1_r_stand.xpm", frogPA, 4 + FROG_R_STAND);
+    loadPixmap("frog1_r_swim.xpm",  frogPA, 4 + FROG_R_SWIM);
 
     Couple frogSize = frogPA.getImageSize();
 
@@ -316,18 +252,18 @@ BatrachiansEngine::BatrachiansEngine(const string &windowManagerCaption,
                     RCouple(), RCouple(), RCouple(0, +2), RCouple(), frogSize);
 
 
-    loadPixmap(splash0_xpm, splashPA, 0);
-    loadPixmap(splash1_xpm, splashPA, 1);
+    loadPixmap("splash0.xpm", splashPA, 0);
+    loadPixmap("splash1.xpm", splashPA, 1);
 
 
-    loadPixmap(tongue0_r2_xpm, tonguePA, 5);
+    loadPixmap("tongue0_r2.xpm", tonguePA, 5);
 
     RCouple collBoxPos(0, 0);
     RCouple collBoxSize = tonguePA.getImageSize() + RCouple(0, 2);
     RSprite *userTongue = new RSprite(tonguePA,
                             RCouple(), RCouple(), RCouple(),
                             collBoxPos, collBoxSize);
-    Uint32 userFrogColor = SDL_MapRGB(theSDLScreen->format, 0xb5, 0x18, 0x5a);
+    SDL_Color userFrogColor = mapRGBA(0xb5, 0x18, 0x5a);
 
     userFrog = new Frog(userFrogSprite, userTongue, &userDigitPA, userFrogColor);
 
@@ -335,7 +271,7 @@ BatrachiansEngine::BatrachiansEngine(const string &windowManagerCaption,
                             RCouple(), RCouple(), RCouple(),
                             collBoxPos, collBoxSize);
 
-    Uint32 computerFrogColor = SDL_MapRGB(theSDLScreen->format, 0xbd, 0xae, 0xce);
+    SDL_Color computerFrogColor = mapRGBA(0xbd, 0xae, 0xce);
 
     compFrog = new Frog(computerFrog, computerTongue, &computerDigitPA, computerFrogColor);
                         // ownership of RSprite objects passed to Frog objects
@@ -343,42 +279,42 @@ BatrachiansEngine::BatrachiansEngine(const string &windowManagerCaption,
 
     /*  Flies:
     */
-    loadPixmap(fly0_r0_xpm, fly0PA, 2);
-    loadPixmap(fly1_r0_xpm, fly1PA, 2);
-    loadPixmap(fly2_r0_xpm, fly2PA, 2);
-    loadPixmap(fly3_r0_xpm, fly3PA, 2);
+    loadPixmap("fly0_r0.xpm", fly0PA, 2);
+    loadPixmap("fly1_r0.xpm", fly1PA, 2);
+    loadPixmap("fly2_r0.xpm", fly2PA, 2);
+    loadPixmap("fly3_r0.xpm", fly3PA, 2);
 
 
     /*  Crosshairs:
     */
-    loadPixmap(crosshairs_xpm, crosshairsPA, 0);
+    loadPixmap("crosshairs.xpm", crosshairsPA, 0);
     crosshairs = new RSprite(crosshairsPA,
                         RCouple(), RCouple(), RCouple(), RCouple(), RCouple());
 
 
     /* Digits:
     */
-    loadPixmap(digit_user_0_xpm, userDigitPA, 0);
-    loadPixmap(digit_user_1_xpm, userDigitPA, 1);
-    loadPixmap(digit_user_2_xpm, userDigitPA, 2);
-    loadPixmap(digit_user_3_xpm, userDigitPA, 3);
-    loadPixmap(digit_user_4_xpm, userDigitPA, 4);
-    loadPixmap(digit_user_5_xpm, userDigitPA, 5);
-    loadPixmap(digit_user_6_xpm, userDigitPA, 6);
-    loadPixmap(digit_user_7_xpm, userDigitPA, 7);
-    loadPixmap(digit_user_8_xpm, userDigitPA, 8);
-    loadPixmap(digit_user_9_xpm, userDigitPA, 9);
+    loadPixmap("digit_user_0.xpm", userDigitPA, 0);
+    loadPixmap("digit_user_1.xpm", userDigitPA, 1);
+    loadPixmap("digit_user_2.xpm", userDigitPA, 2);
+    loadPixmap("digit_user_3.xpm", userDigitPA, 3);
+    loadPixmap("digit_user_4.xpm", userDigitPA, 4);
+    loadPixmap("digit_user_5.xpm", userDigitPA, 5);
+    loadPixmap("digit_user_6.xpm", userDigitPA, 6);
+    loadPixmap("digit_user_7.xpm", userDigitPA, 7);
+    loadPixmap("digit_user_8.xpm", userDigitPA, 8);
+    loadPixmap("digit_user_9.xpm", userDigitPA, 9);
 
-    loadPixmap(digit_computer_0_xpm, computerDigitPA, 0);
-    loadPixmap(digit_computer_1_xpm, computerDigitPA, 1);
-    loadPixmap(digit_computer_2_xpm, computerDigitPA, 2);
-    loadPixmap(digit_computer_3_xpm, computerDigitPA, 3);
-    loadPixmap(digit_computer_4_xpm, computerDigitPA, 4);
-    loadPixmap(digit_computer_5_xpm, computerDigitPA, 5);
-    loadPixmap(digit_computer_6_xpm, computerDigitPA, 6);
-    loadPixmap(digit_computer_7_xpm, computerDigitPA, 7);
-    loadPixmap(digit_computer_8_xpm, computerDigitPA, 8);
-    loadPixmap(digit_computer_9_xpm, computerDigitPA, 9);
+    loadPixmap("digit_computer_0.xpm", computerDigitPA, 0);
+    loadPixmap("digit_computer_1.xpm", computerDigitPA, 1);
+    loadPixmap("digit_computer_2.xpm", computerDigitPA, 2);
+    loadPixmap("digit_computer_3.xpm", computerDigitPA, 3);
+    loadPixmap("digit_computer_4.xpm", computerDigitPA, 4);
+    loadPixmap("digit_computer_5.xpm", computerDigitPA, 5);
+    loadPixmap("digit_computer_6.xpm", computerDigitPA, 6);
+    loadPixmap("digit_computer_7.xpm", computerDigitPA, 7);
+    loadPixmap("digit_computer_8.xpm", computerDigitPA, 8);
+    loadPixmap("digit_computer_9.xpm", computerDigitPA, 9);
 
 
     /*  Sound effects:
@@ -395,7 +331,7 @@ BatrachiansEngine::BatrachiansEngine(const string &windowManagerCaption,
             return;
         }
 
-        string d = getDir(PKGSOUNDDIR, "PKGSOUNDDIR");
+        static const string d = getDirPathFromEnv(PKGSOUNDDIR, "PKGSOUNDDIR");
 
         try
         {
@@ -423,20 +359,29 @@ BatrachiansEngine::~BatrachiansEngine()
     delete crosshairs;
     delete compFrog;
     delete userFrog;
-    SDL_FreeSurface(cloudSurfaces[0]);
-    SDL_FreeSurface(cloudSurfaces[1]);
-    SDL_FreeSurface(waterSurface);
-    SDL_FreeSurface(skySurface);
-    SDL_FreeSurface(lilyPadSurface);
+    SDL_DestroyTexture(cloudTextures[0]);
+    SDL_DestroyTexture(cloudTextures[1]);
+    SDL_DestroyTexture(waterTexture);
+    SDL_DestroyTexture(skyTexture);
+    SDL_DestroyTexture(lilyPadTexture);
 }
 
 
 ///////////////////////////////////////////////////////////////////////////////
 
 
+void
+BatrachiansEngine::loadPixmap(const char *filePath,
+                              PixmapArray &pa,
+                              size_t index)
+{
+    GameEngine::loadPixmap(pkgPixmapDir + filePath, pa, index);
+}
+
+
 /*virtual*/
 void
-BatrachiansEngine::processKey(SDLKey keysym, bool pressed)
+BatrachiansEngine::processKey(SDL_Keycode keysym, bool pressed)
 {
     controller.processKey(keysym, pressed);
 }
@@ -463,7 +408,7 @@ BatrachiansEngine::tick()
     tickCount++;
 
     if (controller.isFullScreenToggleRequested())
-        setVideoMode(theScreenSizeInPixels, !inFullScreenMode());  // ignore failure
+        setFullScreenMode(!inFullScreenMode());  // ignore failure
 
     if (!gameOver)
     {
@@ -790,17 +735,17 @@ BatrachiansEngine::draw()
         else
             starAlpha = 255 - (msToPlay - FULL_STARS_MS) * 255 / (BEGIN_STARS_MS - FULL_STARS_MS);
 
-        SDL_SetAlpha(starPA.getImage(0), SDL_SRCALPHA, starAlpha);
-        SDL_SetAlpha(starPA.getImage(1), SDL_SRCALPHA, starAlpha);
+        setTextureAlphaMod(starPA.getImage(0), starAlpha);
+        setTextureAlphaMod(starPA.getImage(1), starAlpha);
 
         fillRect(0, 0, SCRNWID, SCRNHT, blackColor);  // background for alpha blending
 
         putSpriteList(stars);
 
-        SDL_SetAlpha(skySurface, SDL_SRCALPHA, alpha);
-        copyPixmap(skySurface, Couple(0, 0));
-        SDL_SetAlpha(waterSurface, SDL_SRCALPHA, alpha);
-        copyPixmap(waterSurface, Couple(0, 390));
+        setTextureAlphaMod(skyTexture, alpha);
+        copyPixmap(skyTexture, Couple(0, 0));
+        setTextureAlphaMod(waterTexture, alpha);
+        copyPixmap(waterTexture, Couple(0, 390));
 
         Uint8 lilyPadAlpha;
         if (msToPlay > DARK_MS)
@@ -808,14 +753,14 @@ BatrachiansEngine::draw()
         else
             lilyPadAlpha = msToPlay * (255 - 63) / DARK_MS + 63;
 
-        SDL_SetAlpha(lilyPadSurface, SDL_SRCALPHA, lilyPadAlpha);
-        copyPixmap(lilyPadSurface, lilyPadSurfacePos);
+        setTextureAlphaMod(lilyPadTexture, lilyPadAlpha);
+        copyPixmap(lilyPadTexture, lilyPadSurfacePos);
 
-        SDL_SetAlpha(cloudSurfaces[0], SDL_SRCALPHA, alpha);
-        SDL_SetAlpha(cloudSurfaces[1], SDL_SRCALPHA, alpha);
+        setTextureAlphaMod(cloudTextures[0], alpha);
+        setTextureAlphaMod(cloudTextures[1], alpha);
 
-        copyPixmap(cloudSurfaces[0], Couple(SCRNWID * 20 / 100, 50));
-        copyPixmap(cloudSurfaces[1], Couple(SCRNWID * 60 / 100, SCRNHT / 4));
+        copyPixmap(cloudTextures[0], Couple(SCRNWID * 20 / 100, 50));
+        copyPixmap(cloudTextures[1], Couple(SCRNWID * 60 / 100, SCRNHT / 4));
     }
 
 
@@ -994,7 +939,7 @@ BatrachiansEngine::moveFlies()
 
         for (size_t j = 0; j < numNewFlies; ++j)
         {
-            size_t type = rand() % 4;
+            size_t type = (size_t) rand() % 4;
             PixmapArray *pa = NULL;
             switch (type)
             {
