@@ -1,8 +1,7 @@
-/*  $Id: BatrachiansEngine.h,v 1.39 2017/12/09 18:26:16 sarrazip Exp $
-    BatrachiansEngine.h - Main engine
+/*  BatrachiansEngine.h - Main engine
 
     batrachians - A fly-eating frog game.
-    Copyright (C) 2004-2011 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2004-2024 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -48,7 +47,8 @@ public:
                         size_t maxNumFlies,
                         bool useSound,
                         bool fullScreen,
-                        bool processActiveEvent);
+                        bool processActiveEvent,
+                        bool useAcceleratedRendering);
     /*
         See base class.
 
@@ -61,16 +61,10 @@ public:
     */
 
     virtual ~BatrachiansEngine();
-    /*
-        Nothing interesting.
-    */
 
-    virtual void processKey(SDLKey keysym, bool pressed);
-    /*
-        Inherited.
-    */
+    virtual void processKey(SDL_Keycode keysym, bool pressed) override;
 
-    virtual void processActivation(bool appActive);
+    virtual void processActivation(bool appActive) override;
     /*  Called when the app is activated or deactivated (iconified).
         'appActive' is true if the app has just become active,
         or false if it has just become inactive.
@@ -84,10 +78,7 @@ public:
         Inherited from GameEngine.
     */
 
-    virtual bool tick();
-    /*
-        Inherited.
-    */
+    virtual bool tick() override;
 
 private:
 
@@ -146,16 +137,16 @@ private:
     Uint32 totalPauseTime;  // total ms spent in pause mode
     Uint32 pauseStartTime;  // time in ms when most recent pause started
 
-    Uint32 blackColor;
+    SDL_Color blackColor;
 
-    SDL_Surface *lilyPadSurface;
+    SDL_Texture *lilyPadTexture;
     Couple lilyPadSurfacePos;
     LilyPad leftLilyPad;
     LilyPad rightLilyPad;
 
-    SDL_Surface *skySurface;
-    SDL_Surface *waterSurface;
-    SDL_Surface *cloudSurfaces[2];
+    SDL_Texture *skyTexture;
+    SDL_Texture *waterTexture;
+    SDL_Texture *cloudTextures[2];
 
     PixmapArray starPA;
     RSpriteList stars;
@@ -208,6 +199,8 @@ private:
 
 private:
 
+    void loadPixmap(const char *filePath, PixmapArray &pa, size_t index);
+
     void setTicksBeforeNextFly()
     {
         ticksBeforeNextFly = 1 + 0 * FPS + rand() % (2 * FPS);
